// -- this file should link together the creator, manipulator, display and file handler to provide the user with all the methods
// -- required to build and run simulations

package shared.model;

import shared.model.storage.*;
import shared.model.display.*;
import shared.model.structure.*;
import shared.model.reader.*;

import java.util.*;
import javax.vecmath.*;

public class SimulationModel
{
        SimulationStore storage;
        SimulationCreator creator;
        SimulationManipulator manipulator;
        FileHandler fileHandler;

        Display display;


        public SimulationModel()
        {
                storage = new SimulationStore();
                creator = new SimulationCreator(storage);
                manipulator = new SimulationManipulator(storage);
                display = new Display();
                fileHandler = new FileHandler();
        }

        ////// system methods //////
        /*
        public void setStorage(SimulationStore s)
        {
                storage = s;
        }
        */
        public SimulationStore getStorage()
        {
                return storage;
        }

        public void addTemplate(Object o)
        {
                creator.addTemplate(o);
        }

        public void createObjectInstance(SimObject object)
        {
                creator.createObjectInstance(object);
        }

        public void putInstance(String hashValue, Object object)
        {
                storage.putInstance(hashValue, object);
        }

        public Object getInstance(String hashValue)
        {
                return storage.getInstance(hashValue);
        }

        public void putTemplate(String hashValue, Object object)
        {
                storage.putTemplate(hashValue, object);
        }

        public Object getTemplate(String hashValue)
        {
                return storage.getTemplate(hashValue);
        }

        public void printInstanceHashMap()
        {
                storage.printInstanceHashMap();
        }

        public void printTemplateHashMap()
        {
                storage.printTemplateHashMap();
        }

        //////////////////////////////////////////////////////////////////

        public String[] getAllParticles()
        {
                return manipulator.getAllParticleIds();
        }

        public String[] getAllObjects()
        {
                return manipulator.getAllObjectIds();
        }


        public void display()
        {
                String[] particles = getAllParticles();
                display.initialiseParticles(particles);

                for (int i=0; i<particles.length; i++)
                {
                        Particle p = (Particle) (storage.getInstance(particles[i]));
                        ParticleTemplate pt = (ParticleTemplate) (storage.getTemplate(p.getTemplateId()));

                        // set the particle shape and colour
                        display.setPrimitive(particles[i], pt.getShapeName(), pt.getShapeDimensions());
                        display.setColour(particles[i], pt.getStateColour(p.getCurrentState()));

                        // set the centre of mass
                        display.setCentreOfMass(particles[i], p.getCentreOfMass());
                }

                display.showParticles(particles);
        }

        public void setSimulationSize(float s)
        {
                manipulator.setSimulationSize(s);
                display.setScale(10f / s);
        }

        // File handling
        public void writeTemplateFile(String filename)
        {
                fileHandler.toFile(filename, manipulator.getAllTemplates());
        }

        public void loadFile(String filename)
        {
                Vector objects = fileHandler.fromFile(filename);

                for (int i=0; i<objects.size(); i++)
                {
                        Object o = (objects.elementAt(i));
                        if (o instanceof ParticleTemplate ||
                            o instanceof SimObjectTemplate ||
                            o instanceof InteractionTemplate ||
                            o instanceof StateTemplate ||
                            o instanceof ConditionTemplate ||
                            o instanceof PotentialTemplate)
                        {
                                addTemplate(o);
                        }
                        else if (o instanceof SimObject)
                        {
                                createObjectInstance((SimObject) o);
                        }
                }
                storage.printInstanceHashMap();
                storage.printTemplateHashMap();
        }

        //////////////////////////////////////////////////////////
        // -- user methods (ie. available to the update algorithm)

        public String[] getRootObjects()
        {
                int num = creator.getNumObjects();
                String[] hashes = new String[num];
                for (int i=0; i<num; i++)
                {
                        hashes[i] = "O"+i;
                }
                return hashes;
        }

        public String getRootObject(int num)
        {
                if (num < creator.getNumObjects())
                {
                        return "O"+num;
                }
                return "";
        }

        // -- operations on a single particle

        public float getParticleRadius(String id)
        {
                return manipulator.getParticleRadius(id);
        }

        public void translateParticle(String id, Vector3f trans)
        {
                Vector3f com = manipulator.translateParticle(id, trans);
                display.setCentreOfMass(id, com);
        }

        public Vector3f getParticleCentreOfMass(String id)
        {
                return manipulator.getParticleCentreOfMass(id);
        }

        public void setParticleCentreOfMass(String id, Vector3f com)
        {
                manipulator.setParticleCentreOfMass(id, com);
                display.setCentreOfMass(id, com);
        }

        public AxisAngle4f getParticleOrientation(String id)
        {
                return manipulator.getParticleOrientation(id);
        }

        public void setParticleOrientation(String id, AxisAngle4f orient)
        {
                manipulator.setParticleOrientation(id, orient);
                display.setOrientation(id, orient);
        }

        public float getParticleMass(String id)
        {
                return manipulator.getParticleMass(id);
        }

        // -- operations on multiple particles
        public String[] getAllParticlesOfObject(String id)
        {
                return manipulator.getAllParticlesOfObject(id);
        }

        // -- boundary operations
        public Vector3f getPosition(Vector3f pos)
        {
                return manipulator.getPosition(pos);
        }

        public boolean testPosition(Vector3f pos)
        {
                return manipulator.testPosition(pos);
        }

        public float getSquaredDistance(Vector3f vect1, Vector3f vect2)
        {
                return manipulator.getSquaredDistance(vect1, vect2);
        }

        public float getDistance(Vector3f vect1, Vector3f vect2)
        {
                return manipulator.getDistance(vect1, vect2);
        }
}